#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "time.h"
#include "timeUtils.h"

int getMonthDayCount(int month, int year)
{
	assert(month <= 12 && month >= 1);

	switch (month)
	{
	case 4:
	case 6: 
	case 9:
	case 11:
		return 30;
	case 2:
		return isLeapYear(year) ? 29 : 28;
	default:
		return 31;
	}
}

Boolean fillDateTime(int day, int month, int year, int hour, int minute, int gmtOffset, DateTime *time)
{
	assert(time != NULL);

	if (day > 0 && day <= getMonthDayCount(month, year) && month > 0 && month <= 12 && hour >= 0 && hour <= 23 && 
		minute >= 0 && minute <= 59 && gmtOffset >= -12 && gmtOffset <= 12)
	{
		time->day = day;
		time->month = month;
		time->year = year;
		time->hour = hour;
		time->minute = minute;
		time->gmtOffset = gmtOffset;
		return true;
	}
	else
	{
		return false;
	}
}

int getYearDay(DateTime date)
{
	int dayCount = 0;
	int month;
	for (month = 1; month < date.month; month++)
	{
		dayCount += getMonthDayCount(month, date.year); 
	}
	dayCount += date.day - 1;
	return dayCount;
}

TimeSpan dateTimeDifference(DateTime t1, DateTime t2)
{
	TimeSpan result;

	result.days = 0;
	result.hours = 0;
	result.minutes = 0;

	result.minutes = t1.minute - t2.minute;
	if (result.minutes < 0)
	{
		result.hours = -1;
		result.minutes += 60;
	}
	result.hours += (t1.hour - t1.gmtOffset) - (t2.hour - t2.gmtOffset);
	if (result.hours < 0)
	{
		result.days = -1;
		result.hours += 24;
	}
	result.days += getYearDay(t1) - getYearDay(t2);
	return result;
}

void dateTimeToString(DateTime t, char buffer[])
{
	char signBuffer[2] = "";
	if (t.gmtOffset >= 0)
	{
		strcpy(signBuffer, "+");
	}
	//Attenzione: considerata corretta anche la stringa di formato "%d.%d"
	sprintf(buffer, "%02d/%02d/%04dT%02d.%02d%s%d", t.day, t.month, t.year, t.hour, t.minute, signBuffer, t.gmtOffset);	
}

void timeSpanToString(TimeSpan t, char buffer[])
{
	sprintf(buffer, "%dG %02dH %02dM", t.days, t.hours, t.minutes);
}